class RateLimitException(Exception):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return (repr(self.value))

class AzureAPIException(Exception):

    def __str__(self):
        return (repr("Azure API exception."))

class AuthException(Exception):

    def __str__(self):
        return (repr("Azure API Authorization Exception. Please verify your credentials."))

class ResourceNotFound(Exception):

    def __str__(self):
        return (repr("Azure resource not found please check your resource group name, subnet name and vnet name."))


def print_newline(data):
    for ip in data:
        print(ip)


import ipaddress

def generate_next_ip(data, count):
    address_space = data["address_space"]
    all_ips = list(ipaddress.ip_network(address_space).hosts())
    all_ips.pop(0)
    all_ips.pop(0)
    all_ips.pop(0)
    for ip in data["ips"]:
        ip_to_remove = ipaddress.ip_address(ip)
        all_ips.remove(ip_to_remove)
    ips_to_return = []
    for ip in all_ips:
        ips_to_return.append(str(ip))
    return ips_to_return[:count]



from utilities.azure_apis import AzureAPI
from utilities.ip_address_utils import generate_next_ip
class VacantIP():
    def __init__(self, client_id, client_secret, subscription_id, tenant_id,
                 token_url, base_url, api_version, subnet, vnet, resource_group, count):
        self.client_id = client_id
        self.client_secret = client_secret
        self.subscription_id = subscription_id
        self.tenant_id = tenant_id
        self.token_url = token_url
        self.base_url = base_url
        self.subnet = subnet
        self.vnet = vnet
        self.api_version = api_version or "2018-08-01"
        self.resource_group = resource_group
        self.count = count

    def get_vacant_ips(self):
        api = AzureAPI(subscription_id=self.subscription_id, client_secret=self.client_secret, client_id=self.client_id,
                       tenant_id=self.tenant_id, base_url="https://management.azure.com",
                       token_url="https://login.microsoftonline.com/" + self.tenant_id + "/oauth2/token/",
                       api_version=self.api_version, subnet=self.subnet, vnet=self.vnet, resource_group=self.resource_group)
        return generate_next_ip(api.get_interfaces_for_subnet(), self.count)


import requests
import sys
from . import RateLimitException, AzureAPIException, AuthException, ResourceNotFound

class AzureAPI():

    def __init__(self, client_id, client_secret, subscription_id, tenant_id,
                 token_url, base_url, api_version, subnet, vnet, resource_group):
        self.client_id = client_id
        self.client_secret = client_secret
        self.subscription_id = subscription_id
        self.tenant_id = tenant_id
        self.token_url = token_url
        self.base_url = base_url
        self.subnet = subnet
        self.vnet = vnet
        self.api_version = api_version
        self.resource_group= resource_group
        self.network_interface_url = self.base_url+"/subscriptions/"+\
                                     self.subscription_id+"/providers/Microsoft.Network/networkInterfaces/?api-version="+\
                                     self.api_version
        self.subnet_url = self.base_url+"/subscriptions/"+\
                                     self.subscription_id+"/resourceGroups/"+self.resource_group+"/providers/Microsoft.Network/virtualNetworks/"+self.vnet+"/subnets/"+\
                                     self.subnet+"?api-version="+self.api_version

    def get_auth_headers(self):
        url = self.token_url
        data = {
            "resource": self.base_url,
            "client_id":self.client_id,
            "client_secret":self.client_secret,
            "grant_type":"client_credentials"
        }
        content = requests.post(url,data=data,headers={})
        if content.status_code == 200:
            response = content.json()
            headers = {"Authorization":"Bearer "+ response["access_token"], "Content-Type":"application/json"}
            return headers
        elif content.status_code==429:
            raise(RateLimitException("Azure Apis Rate Limited"))
            sys.exit(1)
        elif content.status_code==404:
            raise(ResourceNotFound())
        else:
            raise(AzureAPIException())
            sys.exit(1)

    def get_interfaces_for_subnet(self):
        list_ips = []
        headers = self.get_auth_headers()
        subnet = self.get_address_space_for_subnet(headers)
        url = self.network_interface_url
        while True:
            content = requests.get(url=url, headers=headers)
            if content.status_code == 200:
                nics = content.json()
                for nic in nics["value"]:
                    for ip in nic["properties"]["ipConfigurations"]:
                        if ip["properties"]["subnet"]["id"] == subnet["id"]:
                            list_ips.append(ip["properties"]["privateIPAddress"])
                if "nextLink" in nics:
                    url = nics["nextLink"]
                else:
                    break
            elif content.status_code == 429:
                raise (RateLimitException("Azure Apis Rate Limited"))
                sys.exit(1)
            elif content.status_code == 403:
                raise (AuthException)
            elif content.status_code==404:
                raise(ResourceNotFound())
            else:
                raise (AzureAPIException())
                sys.exit(1)
        return {"ips": list_ips, "address_space": subnet["address_space"]}

    def get_address_space_for_subnet(self, headers):
        content = requests.get(url=self.subnet_url, headers=headers)
        if content.status_code == 200:
            subnet = content.json()
            return {"name":subnet["name"], "id":subnet["id"], "address_space": subnet["properties"]["addressPrefix"]}
        elif content.status_code == 429:
            raise (RateLimitException("Azure Apis Rate Limited"))
            sys.exit(1)
        elif content.status_code == 403:
            raise (AuthException)
        elif content.status_code==404:
            raise(ResourceNotFound())
        else:
            print(content.status_code)
            raise (AzureAPIException())
            sys.exit(1)


#!/usr/bin/env python
#
# Python installation script
# Author - @chowmean

from __future__ import print_function
import os.path
import sys
import setuptools

# Project variables
VER_PROP_FILE = os.path.join(os.path.dirname(__file__), 'version.properties')
REQUIREMENTS_FILE = os.path.join(os.path.dirname(__file__), 'requirements.txt')
CLASSIFIERS = [
    "Programming Language :: Python",
    "Operating System :: OS Independent",
    "Intended Audience :: Developers",
    "Development Status :: 4 - Beta",
    "Environment :: Plugins",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "License :: Other/Proprietary License",
    "Natural Language :: English",
]


# Read version properties file and extract version number.
def get_version():
    try:
        with open(VER_PROP_FILE) as f:
            for line in f.readlines():
                if line.startswith("version="):
                    version = line.lstrip('version=').strip()
                    return version
    except IOError as ioe:
        print(ioe, file=sys.stderr)



# Read requirements.txt file and extract the list of dependency.
def get_install_requirements():
    # read requirements
    requires = []
    try:
        with open(REQUIREMENTS_FILE) as f:
            requires = list(map(lambda l: l.strip(), f.readlines()))
    except IOError as ioe:
        print(ioe, file=sys.stderr)
        sys.exit(1)
    return requires


if __name__ == '__main__':

    with open('README.md', 'r') as f:
        readme = f.read()

    setuptools.setup(
        name="azureips",
        version=get_version(),
        description="azureIPs: Gives unallocated Azure ips. ",
        author="chowmean",
        author_email="gaurav.dev.iiitm@gmail.com",
        url="https://github.com/chowmean/azureips",
        keywords=["azureips","azure","network_interface_ips"],
        install_requires=get_install_requirements(),
        packages=["azureips","azureips.utilities"],
        package_data={'': ['LICENSE','requirements.txt','version.properties']},
        classifiers=CLASSIFIERS,
        long_description=readme,
        long_description_content_type="text/markdown",
        license="Apache-2.0"
    )